{***************************************************************}
{*            RichView wrapper for QuickReport                 *}
{***************************************************************}
{*                                                             *}
{* Copyright (c) 2005-2011                                     *}
{*               Henk van Bokhorst / IVS Systemhouse B.V.      *}
{*             & QBS Software                                  *}
{* All rights reserved                                         *}
{*                                                             *}
{***************************************************************}
{*                                                             *}
{* Unit: RVQRDsgn                                              *}
{*                                                             *}
{* Design-time enhancements of the components                  *}
{*                                                             *}
{* 02/15/2011 First released version                           *}
{*                                                             *}
{***************************************************************}

unit RVQRDsgn;

{$I RVQRCtrls.inc}

interface

uses Messages, Windows, Classes, Controls, StdCtrls, SysUtils, Graphics, Buttons,
     Forms, ExtCtrls, Dialogs, Printers, DB, ComCtrls, RVScroll, RichView, PtblRV, RVReport, RVStyle,
     QRCtrls, QRPrntr, QuickRpt, DesignIntf, DesignEditors, 
     {$IFDEF USEQR3} Qr3Const, {$ENDIF}
     {$IFDEF USEQR4} Qr4Const, {$ENDIF}
     {$IFDEF USEQR5} Qr5Const, {$ENDIF}
     QRExpr;

type
  TQRRVDBStringProperty = class(TStringProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValueList(List: TStrings); virtual; abstract;
    procedure GetValues(Proc: TGetStrProc); override;
  end;

  TQRRVDataFieldProperty = class(TQRRVDBStringProperty)
  public
    function GetDataSetPropName: string; virtual;
    procedure GetValueList(List: TStrings); override;
  end;

procedure Register;

implementation

uses TypInfo, RVQRCtrls, RVQRContentLoader;

function TQRRVDBStringProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paValueList, paSortList, paMultiSelect];
end;

procedure TQRRVDBStringProperty.GetValues(Proc: TGetStrProc);

var
  Loop: Integer;
  Values: TStringList;

begin
  Values := TStringList.Create;
  try
    GetValueList(Values);
    for Loop := 0 to Values.Count - 1 do Proc(Values[Loop]);
  finally
    Values.Free;
  end;
end;

function TQRRVDataFieldProperty.GetDataSetPropName: string;
begin
  Result := 'DataSet'; {<-- do not resource}
end;

procedure TQRRVDataFieldProperty.GetValueList(List: TStrings);

var
  Instance: TComponent;
  PropInfo: PPropInfo;
  DataSet: TDataSet;

begin
  Instance := TComponent(GetComponent(0));
  PropInfo := TypInfo.GetPropInfo(Instance.ClassInfo, GetDataSetPropName);
  if (PropInfo <> nil) and (PropInfo^.PropType^.Kind = tkClass)
  then
  begin
    DataSet := TObject(GetOrdProp(Instance, PropInfo)) as TDataSet;
    {$Warnings Off}     // Deprecated
    if (DataSet <> nil) then DataSet.GetFieldNames(List);
    {$Warnings On}
  end;
end;

{ Register components and property editors }

procedure Register;
begin
  RegisterPropertyEditor(TypeInfo(string), TQRDBRichView, 'DataField', TQRRVDataFieldProperty);
  RegisterPropertyEditor(TypeInfo(string), TQRRichView, 'Text', TRVQRContentEditor);
  RegisterComponents('QReport', [TQRRichView, TQRDBRichView]);
  RegisterNonActiveX([TQRCustomRichView, TQRRichView], axrIncludeDescendants);
end;

end.

